
(* ****************************************************************
*
*     Integrate.m
*
**************************************************************** *)

Begin["Integrate`"]
(* Make parser put special function symbols into System context. *)
System`CosIntegral
System`ExpIntegralEi
System`Erf
System`FresnelC
System`FresnelS
System`HypergeometricPFQ
System`LogIntegral
System`PolyLog
System`SinIntegral
System`SimplifyGamma
System`SimplifyPolyGamma

(* Note: Files implemented in C have been commented out. KBR. *)

(*
main.m and table.m
	are read in by sysinit.m
inputrules.m logcases.m
mainalgorithm.m taylorseries.m
theoremslater.m
	are read in by main.m
*)

 Unprotect[ Integrate ] 

 Integrate[ f_,{x_,xmin_,xmax_}] :=  
  Module[ {answer}, 
    answer = IntegrateG[ f,{x,xmin,xmax}]; 
    answer /; FreeQ[answer,IntegrateG]
  ] /; CondForIntegrate[f,{x,xmin,xmax}]

 CondForIntegrate[ f_,{x_,xmin_,xmax_}] :=  False /;
 Or@@(Not[FreeQ[{Hold[f],xmin,xmax},#]]&/@
 {Integrate[a_,b_], IntegrateG[__]})

 CondForIntegrate[ f_,{x_,xmin_,xmax_}] :=  True /;
 !FreeQ[{xmin,xmax},DirectedInfinity]

 CondForIntegrate[ f_,{x_,xmin_,xmax_}] :=  
   Module[ 
    {answer =
      Module[ {test},
        Off[Power::infy,Power::indet,Infinity::indet,General::indet,
            General::dbyz];
	Unprotect[Integer,Log,E];
	Power[ 0, n_?Negative ] ^:= Indeterminate;
        Power[ 1, c_/;!FreeQ[c,DirectedInfinity] ] ^:= Indeterminate;
        Power[ E, c_/;!FreeQ[c,DirectedInfinity] ] ^:= Indeterminate;
	Log[ c__/;!FreeQ[{c},DirectedInfinity]||!FreeQ[{c},Indeterminate]] = 
		Indeterminate;
        Times[ 0, c_/;!FreeQ[c,Indeterminate] || 
		      !FreeQ[c,DirectedInfinity]] ^:= Indeterminate;
        test = {f/.x->xmin, f/.x->xmax}/.{0^s_:>Indeterminate};    
        On[ Power::infy,Power::indet,Infinity::indet,General::indet,
            General::dbyz]; 
	Integer/:Power[ 0, n_?Negative ] =.;
	Integer/:Power[ 1, c_/;!FreeQ[c,DirectedInfinity] ] =.;
	Integer/:Times[ 0,c_/;!FreeQ[c,Indeterminate] || 
			      !FreeQ[c, DirectedInfinity]] =.;
	Log[ c__/;!FreeQ[{c},DirectedInfinity]||!FreeQ[{c},Indeterminate]] =.;
        E/:Power[ E, c_/;!FreeQ[c,DirectedInfinity] ] =.;
	Protect[Integer,Log,E];
        test
     ]},
   True /; Or@@(!FreeQ[answer,#]&/@{
    DirectedInfinity,ComplexInfinity,Indeterminate,RealInterval})
  ]

 CondForIntegrate[ __ ] :=  False
 
 Attributes[ Integrate ] = { Protected, ReadProtected }
 Attributes[ CondForIntegrate ] = { HoldFirst, ReadProtected }

End[]

